<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output indent="yes" method="xml"/>
  
  <xsl:param name="localdescriptor"/>
  <xsl:param name="link"/>
  <xsl:param name="jar"/>
  <xsl:param name="name"/>
  <xsl:param name="version"/>
  <xsl:param name="releaseDate"/>
  
  <xsl:template match="/">
    <xsl:processing-instruction name="xml-stylesheet">
      <xsl:text>type="text/xsl" </xsl:text>
      <xsl:text>href="repository_renderer.xsl"</xsl:text>
    </xsl:processing-instruction>
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="repository">
    <xsl:element name="repository">
    <xsl:variable name="newProduct" select="document($localdescriptor)/products/product[@name=$name]"/>
    <xsl:if test="$newProduct">
      <xsl:element name="product">
        <xsl:attribute name="name"><xsl:value-of select="$newProduct/@name"/></xsl:attribute> 
        <xsl:attribute name="version"><xsl:value-of select="$newProduct/@version"/></xsl:attribute>
        <xsl:attribute name="directory"><xsl:value-of select="$newProduct/@directory"/></xsl:attribute>
        <xsl:if test="$link">
          <xsl:attribute name="link"><xsl:value-of select="$link"/></xsl:attribute>
        </xsl:if>
        <xsl:if test="$jar">
          <xsl:element name="jar">
            <xsl:value-of select="$jar"/>
          </xsl:element>
        </xsl:if>
        <xsl:apply-templates select="$newProduct/desc|$newProduct/href|$newProduct/dependencies"/>
	      <xsl:element name="releaseDate">
	        <xsl:value-of select="$releaseDate"/>
	      </xsl:element>
      </xsl:element>
    </xsl:if>
    <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  
  <!--
      copy all
  -->
  <xsl:template match="*">
  	<xsl:copy>
  	  <xsl:copy-of select="@*"/>
  	  <xsl:apply-templates/>
  	</xsl:copy>
  </xsl:template>
  
  <!--
      remove an eventual product with the same name and version
  -->
  <xsl:template match="product[@name=$name and @version=$version]">
  </xsl:template>

  <!--
      when releasing PRO remove PREVIOUS link from the former product
  -->
  <xsl:template match="product[$link='PRO' and @name=$name and @link='PREVIOUS' and not(@version=$version)]">
    <xsl:copy>
      <xsl:copy-of select="@name"/>
      <xsl:copy-of select="@version"/>
      <xsl:copy-of select="@directory"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>
  <!--
      when releasing PRO move fomer PRO to PREVIOUS
  -->
  <xsl:template match="product[$link='PRO' and @name=$name and @link='PRO' and not(@version=$version)]">
	  <xsl:element name="product">
	    <xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute> 
	    <xsl:attribute name="version"><xsl:value-of select="@version"/></xsl:attribute>
	    <xsl:attribute name="directory"><xsl:value-of select="@directory"/></xsl:attribute>
      <xsl:attribute name="link">PREVIOUS</xsl:attribute>
      <xsl:apply-templates/>
	  </xsl:element>
  </xsl:template>
  <!--
      remove the link attributes from a product with a former version
  -->
  <xsl:template match="product[not($link='PRO') and @name=$name and @link=$link and not(@version=$version)]">
    <xsl:copy>
      <xsl:copy-of select="@name"/>
      <xsl:copy-of select="@version"/>
      <xsl:copy-of select="@directory"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>
</xsl:stylesheet>
