/*
 * $Id $
 * 
 * $Date: 2006/11/27 20:21:25 $ $Revision: 1.5 $ $Author: wsliwins $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

import cern.jjar.runner.JJarRunner;
import cern.jjar.runner.RepositoryElement;
import cern.jjar.utils.Logger;

/**
 * Parent class providing base functionality for others specific tasks
 * 
 * @author Lionel Mestre
 * @author Grzegorz Kruk
 * @version $Id: AbstractJJarTask.java,v 1.5 2006/11/27 20:21:25 wsliwins Exp $
 */
public abstract class AbstractJJarTask extends Task implements Logger {

  /**
   * The task's instance variable, representing the failOnError flag. If true it
   * will stop the build if the jjar-function exits with error. Default is true.
   */
  private boolean failonerror = true;
  
  private boolean silent;
  
  private List repositoryElements = new ArrayList();
  
  //
  // -- CONSTRUCTORS -----------------------------------------------
  //

  public AbstractJJarTask() {}

  //
  // -- PUBLIC METHODS -----------------------------------------------
  //

  public void addConfiguredRepository(RepositoryElement repositoryElement) {
    //System.out.println("adding storedescriptor = "+repositoryElement);
    repositoryElements.add(repositoryElement);
  }
  
  public void setFailonerror(boolean failonerror) {
    this.failonerror = failonerror;
  }
  
  public void setSilent(boolean silent) {
    this.silent = silent;
  }
  
  public boolean isSilent() {
    return silent;
  }
  
  protected final RepositoryElement[] getRepositoryElements() {
    return (RepositoryElement[]) repositoryElements.toArray(new RepositoryElement[repositoryElements.size()]);
  }
  
  protected abstract JJarRunner createRunner();
  
  protected abstract void processResult(Object result);
  
  /**
   * Execute the configured task
   * 
   * @throws BuildException BuildExceptions are thrown when required attributes
   *         are missing. Exceptions thrown by Velocity are rethrown as
   *         BuildExceptions.
   */
  public void execute() throws BuildException {
    try {
      JJarRunner runner = createRunner();
      runner.setSilent(isSilent());
      Object result = runner.run();
      
      if (result != null) {
        processResult(result);
      }
      if (!silent) {
        log("Finished");
      }
    } catch (Exception e) {
      if (failonerror) {
        if (!silent) {
          log("Exception occured: " + getStacktrace(e));
        }
        throw new BuildException("Exception occured : " + getStacktrace(e), e);
      } else {
        if (!silent) {
          log("!!!!! A problem occured preventing the task to complete : " + getStacktrace(e));
        }
      }
    }
  }
  
  private String getStacktrace(Exception exc) {
    final StringWriter strWriter = new StringWriter();
    final PrintWriter prWriter = new PrintWriter(strWriter);
    exc.printStackTrace(prWriter);
    final String result = strWriter.toString();
    prWriter.close();
    return result;
  }
  
  
}
