/*
 * $Id $
 * 
 * $Date: 2006/11/16 20:14:03 $ $Revision: 1.21 $ $Author: enes $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar;

import java.io.File;

import cern.jjar.runner.CleanRunner;
import cern.jjar.runner.GetRunner;
import cern.jjar.runner.PublishRunner;
import cern.jjar.runner.RepositoryElement;
import cern.jjar.utils.Logger;

/**
 * JJar command line program
 * 
 * @author Lionel Mestre
 * @version $Id: JJar.java,v 1.21 2006/11/16 20:14:03 enes Exp $
 */
public class JJar {

  private final static String FETCH = "fetch";
  private final static String CLEAN = "clean";
  
  public static void main(String args[]) {
    /*
     * first thing is the verb
     */
    if (args.length < 1) {
      usage();
    }
    String verb = args[0];
    if (verb.equals(FETCH)) {
      fetch(args);
//    } else if (verb.equals(CLEAN)) {
//      clean(args);
    } else {
      usage();
    }

  }

  public static void usage() {
    System.out.println("Usage :");
    System.out.println();
//    System.out.println("  clean [-n number of nonobsolete products to keep] [-p productname]");
//    System.out.println("        [-rr path to the repository to clean] ");
//    System.out.println("		[-path where to create the new repository.xml] ");
//    System.out.println("   - Clean the obsolete versions of the product specified by productname. ");
//    System.out.println();
    System.out.println("  fetch [-ld localDescriptor] [-p product] [-v version]");
    System.out.println("        [-lr localRepository] [-rr remoteRepository]");
    System.out.println("        [-ks] [-kv] [-od]");
    System.out.println("   - Fetch the specified product of the ones defined in the local descriptor");
    System.out.println("     Use either :");
    System.out.println("       -ld localDescriptor  : the pathname of the local descriptor");
    System.out.println("         or");
    System.out.println("       -p product : the name of the product to retrieve");
    System.out.println("       -v version : the version of the package to retrieve");
    System.out.println("                    (the default version by default)");
    System.out.println("     Options :");
    System.out.println("       -lr localRepository  : the pathname of the local repository where to place");
    System.out.println("                              the downloaded jars (. by default)");
    System.out.println("       -rr remoteRepository : the url of the repository where to fetch the available jars");
    System.out.println("     Flags :");
    System.out.println("       -kv  : keep version in the name of the jars fetched from the remote repository");
    System.out.println("             (false by default)");
    System.out.println("       -od  : only dependencies. Do not fetch the target but only the associated dependencies.");
    System.out.println("             (false by default)");
    System.exit(0);
  }

  private static class SysOutLogger implements Logger {
    public void log(String msg) {
      System.out.println(msg);
    }
  }

  public static void clean(String[] args) {
	  if (args.length < 4) {
	      usage();
	      return;
	    }
	  
	  String path = null;
	  String productName = null;
	  int numberOfVersionsToKeep = 5;
    
	  CleanRunner runner = new CleanRunner(new SysOutLogger());
	  for (int i = 1; i < args.length; i++) {
		  String arg = args[i];
		  if (arg.startsWith("-")) {
			  if (arg.equals("-p")) {
				  productName = args[++i];
			  } else if (arg.equals("-n")) {
				  numberOfVersionsToKeep = new Integer(args[++i]).intValue();
			  } else if (arg.equals("-rr")) {
				  runner.setRepositoryElements(new RepositoryElement[]{new RepositoryElement(args[++i])});
			  } else if (arg.equals("-path")) {
				  path = args[++i];
			  } else {
				  System.out.println("!!!! Wrong option : " + arg);
				  System.out.println();
				  usage();
				  return;
			  }
		  }
	  }
    
	  if(productName == null)
		  return;
	  
	  try {
		  runner.setNumberOfVersionsToKeep(numberOfVersionsToKeep);
		  runner.setUpCleanRunner(productName, path);
    	  runner.run();
    	  
    	  //PublishRunner.saveRepositoryFile(runner.getCleanedDocument(), null, new File(newRepositoryFilePath));
	  } catch (Exception e) {
		  System.out.println("Exception occured while running JJAR");
		  e.printStackTrace();
	  }
  }

  public static void fetch(String[] args) {
    if (args.length < 3) {
      usage();
      return;
    }
    GetRunner runner = new GetRunner(new SysOutLogger());
    for (int i = 1; i < args.length; i++) {
      String arg = args[i];
      if (arg.startsWith("-")) {
        if (arg.equals("-p")) {
          runner.setProduct(args[++i]);
        } else if (arg.equals("-v")) {
          runner.setVersion(args[++i]);
        } else if (arg.equals("-lr")) {
          runner.setLocalrepository(args[++i]);
        } else if (arg.equals("-rr")) {
          runner.setRepositoryElements(new RepositoryElement[]{new RepositoryElement(args[++i])});
        } else if (arg.equals("-ld")) {
          runner.setLocaldescriptor(new File(args[++i]));
        } else if (arg.equals("-kv")) {
          runner.setKeepversion(true);
        } else if (arg.equals("-od")) {
          runner.setOnlydependencies(true);
        } else {
          System.out.println("!!!! Wrong option : " + arg);
          System.out.println();
          usage();
          return;
        }
      }
    }
    try {
      runner.run();
    } catch (Exception e) {
      System.out.println("Exception occured while running JJAR");
      e.printStackTrace();
    }
  }
}
