/*
 * $Id $
 * 
 * $Date: 2005/01/10 08:15:59 $ $Revision: 1.4 $ $Author: lmestre $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar;

import java.io.File;

import cern.jjar.runner.JJarRunner;
import cern.jjar.runner.PublishRunner;

/**
 * An ant task for publishing jars
 * 
 *  -<b>publish</b>- publish jars in the development repository. Actually
 * all files (but not directories) are copied to the dev area. Each product has
 * DEV link. <br/>
 * Parameters:
 * <ul>
 * <li><i>remoterepository</i>- URL to the development repository. It has to
 * be passed with file protocol (e.g.: file:////web5.cern.ch/acccodev/)
 * <li><i>localrepository</i>- Path to the directory with jars/zips/...
 * which are to be published
 * <li><i>localdescriptor</i>- path to the file on the local disc (e.g.:
 * c:/projects/bic/project.xml)
 * </ul>
 * 
 * @author Lionel Mestre
 * @author Grzegorz Kruk
 * @version $Id: PublishTask.java,v 1.4 2005/01/10 08:15:59 lmestre Exp $
 */
public class PublishTask extends AbstractJJarTask {

  /* Local descriptor file */
  protected File localdescriptor;
  protected String localrepository;

  //
  // -- CONSTRUCTORS -----------------------------------------------
  //

  public PublishTask() {}

  //
  // -- PUBLIC METHODS -----------------------------------------------
  //

  public void setLocaldescriptor(File localdescriptor) {
    this.localdescriptor = localdescriptor;
  }
  
  /**
   * @param localrepository The localrepository to set.
   */
  public final void setLocalrepository(String localrepository) {
    this.localrepository = localrepository;
  } 

  protected JJarRunner createRunner() {
    PublishRunner runner = new PublishRunner(this);
    runner.setLocaldescriptor(localdescriptor);
    runner.setLocalrepository(localrepository);
    runner.setRepositoryElements(getRepositoryElements());
    return runner;
  }
  
  protected void processResult(Object result) {
  }

}
