/*
 * $Id $
 * 
 * $Date: 2005/01/10 08:15:59 $ $Revision: 1.4 $ $Author: lmestre $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar;

import java.io.File;

import cern.jjar.runner.JJarRunner;
import cern.jjar.runner.VersionUpdateRunner;

/**
 * An ant task for updating the version in the local descriptor. The task can work by increasing the 
 * existing version or by taking a complete new version.
 * 
 * Parameters:
 * <ul>
 * <li><i>localdescriptor</i> - path to the file on the local disc (e.g.: * c:/projects/bic/product.xml)</li>
 * <li><i>increase</i> - the field in the version that have to be increased amongst (major, minor, tiny, modifier)
 * where a version is formatted as major.minor.tiny. For instance in '3.2.4', major is 3, minor is 2 and tiny is 4.
 * </li>
 * <li><i>version</i> - an arbitrary new version.
 * </li>
 * </ul>
 * 
 * @author Lionel Mestre
 * @version $Id: VersionUpdateTask.java,v 1.4 2005/01/10 08:15:59 lmestre Exp $
 */
public class VersionUpdateTask extends AbstractJJarTask {

  /* Local descriptor file */
  private File localdescriptor;

  private String increase;
  private String version;

  //
  // -- CONSTRUCTORS -----------------------------------------------
  //

  public VersionUpdateTask() {}

  //
  // -- PUBLIC METHODS -----------------------------------------------
  //

  public void setLocaldescriptor(File localdescriptor) {
    this.localdescriptor = localdescriptor;
  }
  
  /**
   * @param increase The increase to set.
   */
  public final void setIncrease(String increase) {
    this.increase = increase;
  }

  /**
   * @param version The version to set.
   */
  public final void setVersion(String version) {
    this.version = version;
  }

  protected JJarRunner createRunner() {
    VersionUpdateRunner runner = new VersionUpdateRunner(this);
    runner.setLocaldescriptor(localdescriptor);
    runner.setIncrease(increase);
    runner.setVersion(version);
    return runner;
  }
  
  protected void processResult(Object result) {
  }

}
