/*
 * $Id $
 * 
 * $Date: 2006/11/30 10:42:48 $ $Revision: 1.7 $ $Author: enes $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.repository;

/**
 * @version $Id: Dependency.java,v 1.7 2006/11/30 10:42:48 enes Exp $
 * @author Lionel Mestre
 */
public class Dependency implements Cloneable {

  private Version version = null;
  private String product;
  private boolean local = false;
  private boolean excluded = false;

  public Dependency() {}

  public Dependency(String product, Version v, boolean local, boolean excluded) {
    this.product = product;
    this.version = v;
    this.local = local;
    this.excluded = excluded;
  }

  public Dependency(String product, Version v) {
    this(product, v, false, false);
  }

  /*
   * (non-Javadoc)
   * 
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    try {
      Dependency dependency = (Dependency) super.clone();
      dependency.version = (Version) version.clone();
      return dependency;
    } catch (CloneNotSupportedException e) {
      return this;
    }
  }

  public boolean isLocal() {
    return local;
  }

  public boolean isExcluded() {
    return excluded;
  }

  public void setLocal(boolean local) {
    this.local = local;
  }

  public void setExcluded(boolean excluded) {
    this.excluded = excluded;
  }

  public void setProduct(String product) {
    this.product = product;
  }

  public String getProduct() {
    return product;
  }

  public Version getVersionAsObject() {
    return version;
  }

  public void setVersion(Version v) {
    this.version = v;
  }

  public int hashCode(){
      int productHash = product==null ? 0 : product.hashCode();
      int versionHash = version==null ? 0 : (version.stringValue()).hashCode();
		
      //17 and 37 are contants used to optimize the hashcode. the numbers are arbitrary
      int result = 17;
      result = 37*result + productHash;
      result = 37*result + versionHash;
      return result;
  }
  
  public boolean equals(Object obj) {
    if (!(obj instanceof Dependency)) {
      return false;
    }
    
    Dependency d = (Dependency) obj;
    boolean productEquals = (product == null && d.product == null)
        || (product != null && d.product != null && product.equals(d.product));
    boolean versionEquals = (version == null && d.version == null)
        || (version != null && d.version != null && version.equals(d.version));
    return productEquals && versionEquals;
  }

  public boolean equals(String pName, String pVersion) {
    boolean productNameEquals = (product == null && pName == null) || product.equals(pName);
    boolean versionEquals = (version == null && pVersion == null) || version.equals(pVersion);

    return productNameEquals && versionEquals;
  }

  public String toString() {
    return "Dependency [" + product + ", " + version + ", l(" + local + "), e("+excluded+")]";
  }
}