/*
 * $Id $
 * 
 * $Date: 2006/11/30 10:42:48 $ $Revision: 1.12 $ $Author: enes $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.repository;

import java.net.URL;

import org.w3c.dom.Document;


/**
 * first cut at a repository interface
 * 
 * @author <a href="mailto:geirm@optonline.net">Geir Magnusson Jr.</a>
 * @author Lionel Mestre
 * @version $Id: Repository.java,v 1.12 2006/11/30 10:42:48 enes Exp $
 */
public interface Repository {

  public static final String REPOSITORY_FILE_NAME = "repository.xml";
  
  /**
   * Returns the parsed document of the repository descriptor
   * @return Document
   */
  public Document getDocument();
  
  /**
   * Returns the directory of production repository
   * @return URL
   */
  public URL getProductionDescriptorRootDirURL();
  
  /**
   * Returns the production repository store as defined by method @link RepositoryStore#isProduction()
   * @return production repository store
   */
  public RepositoryStore getProductionStore();
  
  /**
   * Computes a list of all obsolete versions for a given product
   * @param product the product for which to search for obsolete products
   * @return Product[] A list of obsolete products
   */
  public Product[] computeObsoleteVersionsForProduct(Product product);
  
  /**
   * Computes the obsolete versions of a certain product
   * 
   * @param product the product to be cleaned
   * @return
   */
  public Product[] computeObsoleteProducts(Product product);

  /**
   * Returns product with given name and version (version can be both numeral
   * or linked).
   * @param productName the name of the product to get
   * @param version the version of the product to get
   * @return the product matching both the name and the version or null
   */
  public Product getProduct(String productName, Version version);

  /**
   * Returns array with all dependencies of given dependencies.
   * @param dependenciesToResolve the array of dependencies to resolve
   * @param excludeTargets whether or not to include the targets passed in parameter
   * @param defaultVersion the default version to get is no version is specified
   * @return the resolved dependencies after the tree of dependencies has been traversed
   */
  public Dependency[] getDependencies(Dependency[] dependenciesToResolve, boolean excludeTargets, Version defaultVersion);

  /**
   * Returns array with all dependencies of given dependency.
   * @param dependencyToResolve the dependency to resolve
   * @param excludeTarget whether or not to include the target passed in parameter
   * @param defaultVersion the default version to get is no version is specified
   * @return the resolved dependencies after the tree of dependencies has been traversed
   */
  public Dependency[] getDependencies(Dependency dependencyToResolve, boolean excludeTarget, Version defaultVersion);

  /**
   * Returns URL of jar for given product and version.
   * @param productName the name of the product to get
   * @param version the version of the product to get
   * @return the URL of jar for given product and version.
   */
  public URL getProductFileUrl(String productName, Version version);
  
  /**
   * Returns the complete path of the jar from the root of the repository.
   * @param productName the name of the product to get
   * @param version the version of the product to get
   * @return the complete path of the jar from the root of the repository.
   */
  public String getProductFilePath(String productName, Version version);

}