/*
 * $Id $
 * 
 * $Date: 2006/11/30 10:42:48 $ $Revision: 1.9 $ $Author: enes $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.repository;

import java.net.URL;
import java.util.ArrayList;
import org.w3c.dom.Document;

/**
 * first cut at a repository interface
 * 
 * @author <a href="mailto:geirm@optonline.net">Geir Magnusson Jr.</a>
 * @author Lionel Mestre
 * @version $Id: RepositoryStore.java,v 1.9 2006/11/30 10:42:48 enes Exp $
 */
public interface RepositoryStore {
  
  /**
   * Returns all products in this store
   * @return all products in this store
   */
  public MultiVersionsProduct[] getProducts();
  
  /**
   * Returns true if the store is production
   * @return boolean
   */
  public boolean isProduction();
  
  /**
   * Returns the parsed document
   * @return Document the parsed document
   */
  public Document getDocument();
  
  /**
   * Returns the directory to the production repository
   * @return URL
   */
  public URL getStoreRootDirURL();
  
  /**
   * Returns product with given name and version (version can be both numerical or linked).
   * @param productName the name of the product to get
   * @param version the version of the product to get
   * @return the product matching both the name and the version or null
   */
  public Product getProduct(String productName, Version version);

  
  /**
   * Returns all versions for a given project
   * @param productName the productname for which to return all versions
   * @return MultiVersionsProduct all versions of a given project
   */
  public MultiVersionsProduct getProduct(String productName);
  
  /**
   * Returns URL of jar for given product and version.
   * @param product the product the url is to be got for
   * @return the URL of jar for given product
   */
  public URL getProductFileUrl(Product product);
  
  /**
   * Returns path of jar for given product.
   * @param product the product the path is to be got for
   * @return path of jar for given product.
   */
  public String getProductFilePath(Product product);
}