<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="html" version="1.0" encoding="UTF-8" indent="yes"/>
  <xsl:key name="products-by-name" match="product" use="@name"/>
  <xsl:key name="products-by-dependency" match="product" use="dependencies/dep/@product"/>
  <xsl:template match="/">
	<html>
  <head>
    <title>Products available in <xsl:value-of select="repository/@name"/> repository</title>
  </head>
  <body>
    <h1>Products available in <xsl:value-of select="repository/@name"/> repository</h1>
    <ul>
    <xsl:apply-templates select="repository" mode="summary"/>
    </ul>
    <br/>
    <xsl:apply-templates select="repository/product">
      <xsl:sort select="@name"/>
    </xsl:apply-templates>
  </body>
  </html>
  </xsl:template>
  
    <xsl:template match="repository" mode="summary">
      <xsl:for-each select="product[count(. | key('products-by-name', @name)[1]) = 1]">
      <xsl:sort select="@name" />
       <li>
      <b><xsl:value-of select="@name" /></b> -->
      <xsl:for-each select="key('products-by-name', @name)">
        <xsl:sort select="@version" order="descending"/>
        <xsl:if test="not(position()=1)">, </xsl:if>
        <a href="{concat('#',@name, '-', @version)}">
	<xsl:choose>
          <xsl:when test="(@link='PRO') or (@link='PRO,NEXT') or (@link='NEXT,PRO')"><b><xsl:value-of select="@version" />-<xsl:value-of select="@link"/></b></xsl:when>
	  <xsl:otherwise><xsl:value-of select="@version" /><xsl:if test="@link">-<xsl:value-of select="@link"/></xsl:if></xsl:otherwise>
	</xsl:choose>
	</a>
      </xsl:for-each>
      </li>
    </xsl:for-each>
   </xsl:template>
  
  <xsl:template match="product">
  <table>
  <tr>
    <td align="left" valign="top" bgcolor="#CCCCFF" width="50%"><a name="{@name}-{@version}">
<h3><xsl:value-of select="@name"/> (<xsl:value-of select="@version"/><xsl:if test="@link"> - <a name="{@name}-{@link}"><xsl:value-of select="@link"/></a></xsl:if>)</h3></a>
  <ul>
    <xsl:if test="releaseDate"><li><b>Since : </b><font color="#333366"><b><xsl:value-of select="releaseDate"/></b></font></li></xsl:if>
    <li><b>info : </b><xsl:value-of select="desc"/></li>
    <li><b>url : </b><a href="{href}"><xsl:value-of select="href"/></a></li>
    <li><b>directory : </b><a href="{@directory}"><xsl:value-of select="@directory"/></a></li>
    <li><b>file : </b><a href="{concat(@directory,'/',jar)}"><xsl:value-of select="jar"/></a></li>
  </ul>
    </td>
    <td align="left" valign="top" bgcolor="#CCFFCC">
      <table>
        <tr>
          <td align="left">Using<br/>
          <xsl:choose>
          <xsl:when test="dependencies/dep">
          <table border="1" cellpadding="3" cellspacing="0">
          <xsl:apply-templates select="dependencies/dep">
            <xsl:sort select="@product"/>
	        </xsl:apply-templates>
          </table>
          </xsl:when>
          <xsl:otherwise>NONE</xsl:otherwise>
          </xsl:choose>
          </td>
        </tr>
      </table>
    </td>
    <td align="left" valign="top" bgcolor="#FFCCFF">
      <table>
        <tr>
         <td align="left">Used by<br/>
	    <table border="1" cellpadding="3" cellspacing="0">
	      <xsl:call-template name="usedby"/>
	    </table>
         </td>
       </tr>
     </table>
     </td>
  </tr>
  </table>
  <br/>
  </xsl:template>
  
  <xsl:template match="dep">
    <xsl:element name="tr">
      <xsl:if test="@local='true'"><xsl:attribute name="bgcolor">#99CC99</xsl:attribute></xsl:if>
  <td><xsl:value-of select="@product"/></td>
  <td>
  <xsl:choose>
    <xsl:when test="@version">
      <xsl:choose>
        <xsl:when test="key('products-by-name', @product)">
	  <a href="{concat('#',@product, '-', @version)}"><xsl:value-of select="@version"/></a>
        </xsl:when>
	      <xsl:otherwise>
          <a href="{concat('__3rd_PARTY__/repository.xml#',@product, '-', @version)}"><xsl:value-of select="@version"/></a>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:otherwise>default</xsl:otherwise>
  </xsl:choose>
  </td>
  <td>
  <xsl:choose>
    <xsl:when test="@local='true'">local</xsl:when>
    <xsl:otherwise>external</xsl:otherwise>
  </xsl:choose>
  <xsl:if test="@excluded='true'">, excluded</xsl:if> 
  </td>

  </xsl:element>
  </xsl:template>

  <xsl:template name="usedby">
    <xsl:variable name="name" select="@name"/>
    <xsl:variable name="version" select="@version"/>
    <xsl:variable name="link" select="@link"/>
    <xsl:for-each select="key('products-by-dependency', @name)">
      <xsl:sort select="@name"/>
      <xsl:sort select="@version" order="descending"/>
      <xsl:variable name="depVersion" select="dependencies/dep[@product=$name]/@version"/>
      
      <xsl:if test="($depVersion = $version) or ($depVersion = $link)">
        <tr>
			<td><xsl:value-of select="@name"/></td>
			<!--
			To always show the verisonnumber uncomment this line, and remove the "choose"-claus
			<td><a href="{concat('#',@name, '-', @version)}"><xsl:value-of select="@version"/></a></td>
			-->
			<xsl:choose>
				<xsl:when test="@link">
					<td><a href="{concat('#',@name, '-', @version)}"><xsl:value-of select="@link"/></a></td>
				</xsl:when>
				<xsl:otherwise>
					<td><a href="{concat('#',@name, '-', @version)}"><xsl:value-of select="@version"/></a></td>	
				</xsl:otherwise>
			</xsl:choose>
		</tr>
		<xsl:variable name="previousName" select="@name"/>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>
</xsl:stylesheet>
