/*
 * $Id $
 *
 * $Date: 2006/10/30 14:42:38 $
 * $Revision: 1.6 $
 * $Author: enes $
 *
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.runner;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import cern.jjar.repository.Dependency;
import cern.jjar.repository.LocalDescriptorXML;
import cern.jjar.repository.Product;
import cern.jjar.repository.Repository;
import cern.jjar.repository.Version;
import cern.jjar.resources.Resources;
import cern.jjar.utils.Logger;

/**
 * 
 * @deprecated  This will be done by the releaseRunner instead of here
 * 
 * @version $Id: ProductVersionerRunner.java,v 1.6 2006/10/30 14:42:38 enes Exp $
 * @author Lionel Mestre
 */
public class ProductVersionerRunner extends AbstractJJarRunner {

  protected File localdescriptor;
  private boolean resolvealias;

  //
  // -- CONSTRUCTORS -----------------------------------------------
  //

  /* Creates a new instance of PublishRunner */
  public ProductVersionerRunner(Logger logger) {
    super(logger);
  }

  //
  // -- PUBLIC METHODS -----------------------------------------------
  //

  public final void setResolvealias(boolean resolvealias) {
    this.resolvealias = resolvealias;
  }

  /**
   * @param localdescriptor The localdescriptor to set.
   */
  public final void setLocaldescriptor(File localdescriptor) {
    this.localdescriptor = localdescriptor;
  }

  public static void main(String[] args) {
    try {
    	ProductVersionerRunner runner = new ProductVersionerRunner(null);
    	runner.setRepositoryElements(new RepositoryElement[] { 
    	          new RepositoryElement("file://C:\\repositoryXML\\pro\\"),
    	          new RepositoryElement("file://C:\\repositoryXML\\__3rd_PARTY__\\")});
    	runner.setLocaldescriptor(new File("C:\\java\\dev\\accsoft\\commons\\accsoft-commons-shregdiag\\product.xml"));
      
//      runner.setRepositoryElements(new RepositoryElement[]{
//          new RepositoryElement("http://slwww.cern.ch/~pcrops/releaseinfo/pcropsdist"),
//          new RepositoryElement("http://slwww.cern.ch/~pcrops/releaseinfo/pcropsdist/__3rd_PARTY__")});
//      runner.setLocaldescriptor(new File("C:\\lionel\\cvs\\cmmnbuild\\cern-jjar\\product.xml"));
  
    	runner.run();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  //
  // -- implements Runnable -----------------------------------------------
  //

  public Object run() throws Exception {
    checkParameters();
    Version defaultVersion = Version.valueOf(defaultdependencyversion);
    Repository repository = lookupRepository();
    LocalDescriptorXML localDescriptorXML = new LocalDescriptorXML(localdescriptor);
    String dependencies = generateDependenciesFragment(repository, localDescriptorXML, defaultVersion);
    File dependenciesFile = new File(localdescriptor.getParentFile(), "_dependencies_.xml");
    BufferedWriter bw = new BufferedWriter(new FileWriter(dependenciesFile));
    bw.write(dependencies, 0, dependencies.length());
    bw.close();
    registerProduct(dependenciesFile);
    dependenciesFile.delete();
    return null;
  }

  //
  // -- PROTECTED METHODS -----------------------------------------------
  //

  //
  // -- PRIVATE METHODS -----------------------------------------------
  //

  private void checkParameters() throws Exception {
    if (getRepositoryElement() == null)
      throw new Exception("Remote repository has to be set!");
    if (localdescriptor == null)
      throw new Exception("Local descriptor has to be set!");
  }

  private String generateDependenciesFragment(Repository repository, LocalDescriptorXML localDescriptorXML,
      Version defaultVersion) {
    StringBuffer sb = new StringBuffer();
    Product[] products = localDescriptorXML.getProducts();
    sb.append("<products>\n");
    for (int i = 0; i < products.length; i++) {
      Product product = products[i];
      Dependency[] dependencies = product.getDependenciesAsArray();
      sb.append("  <product name=\"");
      sb.append(product.getName());
      sb.append("\">\n");
      sb.append("    <dependencies>\n");
      for (int j = 0; j < dependencies.length; j++) {
        Dependency d = dependencies[j];
        Version v = d.getVersionAsObject() == null ? defaultVersion : d.getVersionAsObject();
        Product dp = repository.getProduct(d.getProduct(), v);
        if (dp != null) {
          sb.append("      <dep product=\"");
          sb.append(dp.getName());
          sb.append("\" version=\"");
          if (resolvealias || !dp.hasAlias()) {
            sb.append(dp.getVersion().stringValue());
          } else {
            sb.append(dp.getAlias(defaultdependencyversion).stringValue());
          }
          if (d.isLocal()) {
            sb.append("\" local=\"true");
          }
          if (d.isExcluded()) {
            sb.append("\" excluded=\"true");
          }
          sb.append("\"/>\n");
        }
      }
      sb.append("    </dependencies>\n");
      sb.append("  </product>\n");
    }
    sb.append("</products>\n");
    return sb.toString();
  }

  protected final void registerProduct(File dependenciesFile) throws TransformerException {
    Transformer t = Resources.getVersionProductTemplate().newTransformer();
    t.setParameter("dependenciesFile", dependenciesFile.getAbsolutePath());

    File output = new File(localdescriptor.getParentFile(), "_" + localdescriptor.getName());
    File oldProductFile = new File(localdescriptor.getParentFile(), localdescriptor.getName() + ".original");
    t.transform(new StreamSource(localdescriptor), new StreamResult(output));
    try {
      Thread.sleep(500);
    } catch (InterruptedException e) {
    }
    localdescriptor.renameTo(oldProductFile);
    try {
      Thread.sleep(300);
    } catch (InterruptedException e1) {
    }
    output.renameTo(localdescriptor);
  }

  // 
  // -- INNER CLASSES -----------------------------------------------
  //

}
