/*
 * $Id $
 *
 * $Date: 2005/04/18 14:18:18 $
 * $Revision: 1.6 $
 * $Author: lmestre $
 *
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.utils;

import cern.jjar.repository.Dependency;
import cern.jjar.repository.Version;

import java.net.JarURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

/**
 *  classpath support for JJar.  This is currently a jumble - will
 *  clean up.
 *
 * @author Geir Magnusson Jr. (original idea from Jakarta)
 * @version $Id: ClasspathUtil.java,v 1.6 2005/04/18 14:18:18 lmestre Exp $ 
 */
public class ClasspathUtil {

  /*
   *  checks to see if the classpath <code>path</code> contains
   *  the package / version
   */
  public static boolean containsVersionedJar(String path, String pkg, String version) {
    List jars = getJarList(path);
    // now, with this set of jars, see if we can find those that match
    // the package name
    Iterator i = jars.iterator();
    while (i.hasNext()) {
      String jar = (String) i.next();
      try {
        URL url = new URL("jar:file:" + jar + "!/");
        JarURLConnection conn = (JarURLConnection) url.openConnection();
        if (packageMatch(conn, pkg, version)) {
          // System.out.println( jar + " : pkg = " + pkg + " ver = " + version );
          return true;
        }
      } catch (Exception e) {
        //System.out.println("Exception (continuing) : " + e );
      }
    }
    return false;
  }

  /*
   *  returns a JJARPackage( info ) for a given jar
   */
  private static Dependency getPackage(String jarname) throws Exception {
    URL url = new URL("jar:file:" + jarname + "!/");
    JarURLConnection conn = (JarURLConnection) url.openConnection();
    return getPackage(conn);
  }

  private static Dependency getPackage(JarURLConnection jarConn) throws Exception {
    if (jarConn == null)
      return null;
    Manifest mfst = jarConn.getManifest();
    if (mfst == null) {
      return null;
    }
    /*
     * get the attributes we care about
     */
    Attributes att = mfst.getMainAttributes();
    String jarpkg = att.getValue(Attributes.Name.IMPLEMENTATION_TITLE);
    String jarver = att.getValue(Attributes.Name.IMPLEMENTATION_VERSION);
    if (jarpkg == null || jarver == null) {
      /*
       * unusable
       */
      return null;
    }
    return new Dependency(jarpkg, Version.valueOf(jarver), false, false);
  }

  /*
   *  determines if two packages are equal
   */
  private static boolean packageMatch(JarURLConnection jarConn, String pkg, String version) throws Exception {
    Dependency jjarp = getPackage(jarConn);
    if (jjarp == null)
      return false;
    return jjarp.equals(pkg, version);
  }

  /*
   *  returns a list of jars in the specified claspath.
   *  will check the system classpath by default
   */
  private static List getJarList(String path) {
    if (path == null) {
      path = "java.class.path";
    }
    String classpath = System.getProperty(path);
    String pathSep = System.getProperty("path.separator");
    /*
     *  take the classpath, and look for jars
     */
    StringTokenizer st = new StringTokenizer(classpath, pathSep);
    ArrayList jars = new ArrayList();
    while (st.hasMoreTokens()) {
      String foo = st.nextToken();
      if (foo.indexOf("jar") != -1)
        jars.add(foo);
    }
    return jars;
  }
}
