/* * $Id $
 * 
 * $Date: 2006/05/15 14:49:26 $ $Revision: 1.4 $ $Author: lmestre $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar;

import java.io.File;

import org.apache.tools.ant.Project;

import cern.jjar.runner.GetRunner;
import cern.jjar.runner.JJarRunner;
import cern.jjar.utils.JJarUtils;

/**
 * An ant task for listing jars.
 * 
 * <br/><br/>Commands description: <br/>-<B>getJars</B>-
 * <ul>
 * <li><i>localdescriptor</i>- path to the file on the local disc (e.g.:
 * c:/projects/bic/project.xml)
 * <li><i>onlydependencies</i>- if true - the project jars are not
 * dowloaded (only dependent products). The default value is <i>true</i>
 * <li><i>product</i>- Name of the product for which dependences jars
 * should be fetched. This property should be passed if there is no local
 * descriptor file (localdescriptor property is not set).
 * <li><i>version</i>- Version of product mentioned above. The default
 * value is PRO (production version).
 * <li><i>defaultdependencyversion</i>- name of default version, when
 * ommited in dependency definition.
 * </ul>
 * 
 * @author Lionel Mestre
 * @author Grzegorz Kruk
 * @version $Id: ListTask.java,v 1.4 2006/05/15 14:49:26 lmestre Exp $
 */
public class ListTask extends AbstractJJarTask {

  private String resultpaths;

  /* Local descriptor file */
  private File localdescriptor;

  /* Name of the product */
  private String product;

  /* The directory in which are located the libraries */
  private String libdirectory;

  /* Keep version while fetching jars */
  private boolean keepversion;

  /* Version of the product */
  private String version;

  /* Indicates if product should be fetched together with its dependencies */
  private boolean onlydependencies = true;

  /**/
  private String defaultdependencyversion = "PRO";

  /**/
  private String separator = ",";

  private boolean resolvealias;

  //
  // -- CONSTRUCTORS -----------------------------------------------
  //

  public ListTask() {}

  //
  // -- PUBLIC METHODS -----------------------------------------------
  //

  public void setResultpaths(String pathrefid) {
    this.resultpaths = pathrefid;
  }

  public void setLocaldescriptor(File localdescriptor) {
    this.localdescriptor = localdescriptor;
  }

  public void setProduct(String product) {
    this.product = product;
  }

  public void setLibdirectory(String libdirectory) {
    this.libdirectory = libdirectory;
  }

  public void setVersion(String version) {
    this.version = version;
  }

  public void setOnlydependencies(boolean onlydependencies) {
    this.onlydependencies = onlydependencies;
  }

  public void setDefaultdependencyversion(String defaultVersion) {
    this.defaultdependencyversion = defaultVersion;
  }
  
  public void setSeparator(String separator) {
    this.separator = separator;
  }
  
  public void setKeepversion(boolean keepversion) {
    this.keepversion = keepversion;
  }

  /**
   * @param resolvealias The resolvealias to set.
   */
  public final void setResolvealias(boolean resolvealias) {
    this.resolvealias = resolvealias;
  }

  protected JJarRunner createRunner() {
    GetRunner runner = new GetRunner(this);
    runner.setKeepversion(keepversion);
    runner.setSimulated(true);
    runner.setLocaldescriptor(localdescriptor);
    runner.setOnlydependencies(onlydependencies);
    runner.setResolvealias(resolvealias);
    runner.setProduct(product);
    runner.setVersion(version);
    runner.setDefaultdependencyversion(defaultdependencyversion);
    runner.setRepositoryElements(getRepositoryElements());
    return runner;
  }

  protected void processResult(Object result) {
    String[] addedPaths = (String[]) result;
    if (resultpaths != null && addedPaths.length > 0) {
      StringBuffer resultValue = new StringBuffer();
      for (int i = 0; i < addedPaths.length; i++) {
        String path = addedPaths[i];
        if (libdirectory != null && libdirectory.length() > 0) {
          File file = new File(path);
          path = JJarUtils.concatenatePath(libdirectory, file.getName());
        }
        resultValue.append(path);
        if (i != addedPaths.length-1) {
          resultValue.append(separator);
        }
      }
      Project project = getProject();
      project.setProperty(resultpaths, resultValue.toString());
    }
  }

}
