/*
 * $Id $
 * 
 * $Date: 2005/01/10 08:15:59 $ $Revision: 1.3 $ $Author: lmestre $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar;

import java.io.File;
import java.io.IOException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;

import cern.jjar.repository.LocalDescriptorXML;
import cern.jjar.repository.Product;
import cern.jjar.runner.GetRunner;

/**
 * Updates the classpath property based on the jar available in the lib directory
 * 
 * @author Lionel Mestre
 * @version $Id: UpdateClasspathTask.java,v 1.3 2005/01/10 08:15:59 lmestre Exp $
 */
public class UpdateClasspathTask extends Task {

  /**
   * The task's instance variable, representing the failOnError flag If true it
   * will stop the build if cvs exits with error. Default is false.
   */
  private boolean failOnError = true;

  private String pathrefid;
  private String projectclassesdirectory;
  private String localrepository;
  private File localdescriptor;

  //
  // -- CONSTRUCTORS -----------------------------------------------
  //

  public UpdateClasspathTask() {}

  //
  // -- PUBLIC METHODS -----------------------------------------------
  //

  //
  // -- PUBLIC METHODS -----------------------------------------------
  //

  public void setPathrefid(String pathrefid) {
    this.pathrefid = pathrefid;
  }

  public void setProjectclassesdirectory(String projectclassesdirectory) {
    this.projectclassesdirectory = projectclassesdirectory;
  }

  public void setLocalrepository(String localrepository) {
    this.localrepository = localrepository;
  }

  public void setFailOnError(boolean failOnError) {
    this.failOnError = failOnError;
  }

  public void setLocaldescriptor(File localdescriptor) {
    this.localdescriptor = localdescriptor;
  }

  /**
   * Execute the configured task
   * 
   * @throws BuildException BuildExceptions are thrown when required attributes
   *         are missing. Exceptions thrown by Velocity are rethrown as
   *         BuildExceptions.
   */
  public void execute() throws BuildException {
    LocalDescriptorXML localDescriptorXML;
    try {
      localDescriptorXML = new LocalDescriptorXML(localdescriptor);
    } catch (IOException e) {
      throw new BuildException(e.getMessage(), e);
    }
    Product p = localDescriptorXML.getProduct();
    if (p == null) {
      throw new BuildException("The product cannot be read from the local descriptor "+localdescriptor);
    }
    String productDir = p.getDirectory();

    //If the path is created using \ - replace it into /

    productDir = productDir.replace('\\', '/');
    //The file is created to deal with both / and

    String[] dirs = productDir.split("/");
    StringBuffer relativePath = new StringBuffer("/../");
    for (int i = 0; i < dirs.length; i++) {
      relativePath.append("../");
    }

    File rootDir = new File(localrepository + relativePath.toString());

    File localProjectDir = new File(localrepository + GetRunner.PROJECT_DEPENDENCY_DIR);
    String[] projects = localProjectDir.list(null);
    if (pathrefid != null && projects != null && projects.length > 0) {
      // create an antpath
      Path antpath = new Path(getProject());

      for (int i = 0; i < projects.length; i++) {
        String projectName = projects[i].substring(0, projects[i].length()
            - GetRunner.PROJECT_DEPENDENCY_EXTENSION.length());

        projectName = projectName.replace(GetRunner.PROJECT_DIR_SEPARATOR, '/');

        File projectClasses = new File(rootDir, projectName + "/" + projectclassesdirectory);
        log("adding localproject : " + projectName);
        antpath.createPathElement().setPath(projectClasses.getAbsolutePath());
      }
      getProject().addReference(pathrefid, antpath);
    }
  }

}