/*
 * $Id $
 * 
 * $Date: 2006/11/15 16:29:39 $ $Revision: 1.7 $ $Author: enes $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.repository;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Lionel Mestre
 * @version $Id: LocalDescriptorXML.java,v 1.7 2006/11/15 16:29:39 enes Exp $
 */
public class LocalDescriptorXML {

	private JJarParser parser = null;
	private List dependencies = new ArrayList();
	private List products = new ArrayList();

  public LocalDescriptorXML(File localDescriptor) throws IOException {
    load(localDescriptor);
  }
  
  public LocalDescriptorXML(File localDescriptor, boolean loadElement) throws IOException {
	    load(localDescriptor);
  }
  
  /*
   * Loads localdescriptor.
   */
  private void load(File localDescriptor) throws IOException {
    parser = new JJarParser(localDescriptor);
    
    Map storesMap = parser.getProductVersionMap();
    for (Iterator i = storesMap.values().iterator(); i.hasNext();) {
      MultiVersionsProduct pvc = (MultiVersionsProduct) i.next();
      //Get product - one pvc contains only one product
      Product p = (Product) pvc.getProducts()[0];
      products.add(p);
      dependencies.addAll(p.getDependencies());
    }
  }
  
   public Document getDocument(){
	   return parser.getDocument();
   }

  /*
   * Returns collection of products defined in the local descriptor.
   */
  public Product[] getProducts() {
    return (Product[]) products.toArray(new Product[products.size()]);
  }

  /*
   * Returns first product defined in the descriptor.
   */
  public Product getProduct() {
    if (products.size() == 0) {
      return null;
    }
    return (Product) products.get(0);
  }

  /*
   * Returns first product defined in the descriptor.
   */
  public Product getProduct(String name) {
    if (products.size() == 0) {
      return null;
    }
    for (int i = 0; i < products.size(); i++) {
      Product p = (Product) products.get(i);
      if (p.getName().equals(name)) {
        return p;
      }
    }
    return null;
  }

  /*
   * Returns dependencies for all products defined in the local descriptor.
   */
  public Dependency[] getDependencies() {
    Dependency[] deps = new Dependency[dependencies.size()];
    return (Dependency[]) dependencies.toArray(deps);
  }

  /*
   * Returns dependencies for all products defined in the local descriptor.
   * (Returns dependencies for a specific product defined in the local descriptor. to be verified)
   */
  public Dependency[] getDependencies(String productName) {
    Product p = getProduct(productName);
    if (p == null) {
      return new Dependency[0];
    }
    return p.getDependenciesAsArray();
  }

  /*
   * used for tests
   */
  public static void main(String[] args) {
    try {
      LocalDescriptorXML rep = new LocalDescriptorXML(new File("C:\\java\\dev\\accsoft\\commons\\accsoft-commons-shregdiag\\product.xml"));
      Dependency[] dependencies = rep.getDependencies();
      for (int i = 0; i < dependencies.length; i++) {
        System.out.println(dependencies[i].getProduct() + " " + dependencies[i].getVersionAsObject());
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

}