/*
 * $Id $
 * 
 * $Date: 2006/10/30 18:54:54 $ $Revision: 1.8 $ $Author: enes $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.repository;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ArrayList;

/**
 * This is a container class for different versions of one product.
 * 
 * @author Grzegorz Kruk
 */
public class MultiVersionsProduct {

  private Map products = new HashMap();

  /** Holds value of property productName. */
  private String productName;

  /* Creates a new instance of ProductsStore */
  public MultiVersionsProduct(String productName) {
    this.productName = productName;
  }

  /*
   * Adds product to the container. Each product is known by its version number
   * and eventually link (if the product is linked). If a product has two links,
   * it is added once per link.
   */
  public void addProduct(Product product) {
	  products.put(product.getVersion().stringValue(), product);
	  if (product.hasAlias()){
		  ArrayList aliases = (ArrayList)product.getAliases();
		  for(int i=0; i<aliases.size(); i++){
			  products.put(((Version)aliases.get(i)).stringValue(), product);
		  }
	  }
  }
  
  
  public Product[] getProducts() {
    return (Product[]) products.values().toArray(new Product[products.size()]);
  }

  /*
   * Returns product object with given version.
   */
  public Product getProduct(Version v) {
    if (v != null){
      Product product = (Product) products.get(v.stringValue());
    	return product;
    }
    return (Product) products.get(Version.PRODUCTION_VERSION.stringValue());
  }

  /**
   * Getter for property productName.
   * 
   * @return Value of property productName.
   *  
   */
  public String getProductName() {
    return this.productName;
  }

  /**
   * Setter for property productName.
   * 
   * @param productName New value of property productName.
   *  
   */
  public void setProductName(String productName) {
    this.productName = productName;
  }

  /**
   * Setter for property productName.
   * 
   * @param productName New value of property productName.
   *  
   */
  public void setProduct(String productName) {
    setProductName(productName);
  }
}
