/*
 * $Id $
 * 
 * $Date: 2006/11/15 16:29:39 $ $Revision: 1.11 $ $Author: enes $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.repository;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;

/**
 * @author gkruk
 * @author enes
 */
public class Product {

  /** Holds value of property name. */
  private String name;

  /** Holds value of property href. */
  private String href;

  /** Holds value of property description. */
  private String description;

  /** Holds value of property short description. */
  private String shortDescription;

  /** Holds value of property directory. */
  private String directory;
  
  /** Holds value of property version. */
  private Version version;

  /** Holds value of property dependencies. */
  private Collection dependencies = new ArrayList();

  /** Holds values of property link. */
  private ArrayList aliases = new ArrayList();
  
  /** Holds value of property jarName. */
  private String jarName;

  /**
	 * A comparator that can be used to compare versions
	 */
	public static final Comparator PRODUCT_COMPARATOR = new Comparator() {
		public final int compare(Object object1, Object object2) {
			Product p1 = (Product) object1;
			Product p2 = (Product) object2;
			int n = p1.getName().compareTo(p2.getName());
			if (n == 0) {
				return p1.getVersion().compareTo(p2.getVersion());
			}
			return n;
		}
	};
  
  /** Creates a new instance of Product */
  public Product() {}

  /**
   * Getter for property name.
   * 
   * @return Value of property name.
   *  
   */
  public String getName() {
    return this.name;
  }

  /**
   * Setter for property name.
   * 
   * @param name New value of property name.
   *  
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * Getter for property href.
   * 
   * @return Value of property href.
   *  
   */
  public String getHref() {
    return this.href;
  }

  /**
   * Setter for property href.
   * 
   * @param href New value of property href.
   *  
   */
  public void setHref(String href) {
    this.href = href;
  }

  /**
   * Getter for property description.
   * 
   * @return Value of property description.
   *  
   */
  public String getDescription() {
    return this.description;
  }

  /**
   * Setter for property description.
   * 
   * @param description New value of property description.
   *  
   */
  public void setDescription(String description) {
    this.description = description;
  }

  /**
   * Getter for property projectName.
   * 
   * @return Value of property projectName.
   *  
   */
  public String getDirectory() {
    return directory;
  }

  public void setDirectory(String directory) {
    this.directory = directory;
  }
  
  public String toString() {
    return "Product [" + name + ", " + description + ", " + href + ", " + version + ", " + getAliasesAsString() + ", " + jarName + "]";
  }

  /**
   * Getter for property version.
   * 
   * @return Value of property version.
   *  
   */
  public Version getVersion() {
    return version;
  }

  /**
   * Setter for property version.
   * 
   * @param version New value of property version.
   *  
   */
  public void setVersion(Version version) {
    this.version = version;
  }

  /**
   * Getter for property dependencies.
   * 
   * @return Value of property dependencies.
   *  
   */
  public Collection getDependencies() {
    return dependencies;
  }

  public Dependency[] getDependenciesAsArray() {
    Dependency[] deps = new Dependency[dependencies.size()];
    return (Dependency[]) dependencies.toArray(deps);
  }

  public void addDependency(Dependency d) {
    if (d.getProduct() == null)
      throw new RuntimeException("Dependency product name has been not defined in product: " + name);
    dependencies.add(d);
  }

  /**
   * Getter for property link.
   * 
   * @return String the alias(es) 
   *  
   */
  public String getAliasesAsString(){
	  String alias = "";
	  
	  for(int i=0;i<aliases.size();i++){
		  if(!"".equals(alias))
			  alias = alias + ",";
		  alias = alias + (Version)aliases.get(i);
	  }
	  return alias;
  }
  
  
  /**
   * Getter for property link.
   * 
   * @return Version The version equal to the defaultdependencyversion if it exists. 
   * Otherwise it returns another alias 
   *  
   */
  public Version getAlias(String version) {
	  Version alias = null;
	  
	  for(int i=0;i<aliases.size();i++){
		  if(((Version)aliases.get(i)).stringValue().equals(version))
			  return (Version)aliases.get(i);
		  else
			  alias = (Version)aliases.get(i);
	  }
	  return alias;
  }
  
  
  /**
   * Getter for property link.
   * 
   * @return collection A collection of the aliases.
   *  
   */
  public Collection getAliases(){
	  return aliases;	  
  }
  
  /**
   * Check if there is an alias equal to a given version.
   * @param version Version to compare with
   * 
   * @return boolean
   *  
   */
  public boolean hasAlias(Version version){
	  if(aliases.contains(version))
		 return true;
	  return false;
  }
  
  
  /**
   * Check if the product has an alias
   * 
   * @return boolean
   *  
   */
  public boolean hasAlias(){
	  if(aliases.size() > 0){
			  return true;
	  }
	  return false;
  }
  
  /**
   * Setter for property alias.
   * 
   * @param aliases New value of property link.
   *  
   */
  public void setAliases(ArrayList aliases){
	  this.aliases = aliases; 
  }
  public void addAlias(Version alias){
	  aliases.add(alias);
  }

  /**
   * Getter for property jarName.
   * 
   * @return Value of property jarName.
   *  
   */
  public String getJarName() {
    return this.jarName;
  }

  /**
   * Setter for property jarName.
   * 
   * @param jarName New value of property jarName.
   *  
   */
  public void setJarName(String jarName) {
    this.jarName = jarName;
  }

  public void setShortDescription(String shortDescription) {
    this.shortDescription = shortDescription;
  }
  
  public String getShortDescription() {
    return shortDescription;
  }
}