/*
 * $Id $
 * 
 * $Date: 2006/11/30 10:42:48 $ $Revision: 1.15 $ $Author: enes $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.repository;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.ArrayList;

import org.w3c.dom.Document;

import cern.jjar.utils.JJarUtils;

/**
 * @author gkruk
 */
public class RepositoryXMLStore implements RepositoryStore {

  public static final String FILE = "file";
  public JJarParser parser = null;

  private String prefixDirectory;

  /** Holds value of property storeRootDirURL. */
  private String storeRootDir;
  private URL storeRootDirURL;

  private String repositoryDescriptor = null;
  private URL repositoryDescriptorURL;
  private File repositoryDescriptorFile;
  private boolean repositoryDescriptorIsLocal = false;

  private Map productsMap = new HashMap();

  public RepositoryXMLStore(String repositoryDir) throws IOException {
    this.repositoryDescriptor = JJarUtils.concatenatePath(repositoryDir, Repository.REPOSITORY_FILE_NAME);
    //Add / at the end if needed
    this.storeRootDir = JJarUtils.concatenatePath(repositoryDir, "");
    load();
  }

  public MultiVersionsProduct[] getProducts() {
    return (MultiVersionsProduct[]) productsMap.values().toArray(new MultiVersionsProduct[productsMap.size()]);
  }
  
  public Product getProduct(String productName, Version v) {
    MultiVersionsProduct pvc = (MultiVersionsProduct) productsMap.get(productName);
    if (pvc != null) {
    	Product product = pvc.getProduct(v);
      return product;
    }
    return null;
  }
  
  public MultiVersionsProduct getProduct(String productName) {
	    MultiVersionsProduct pvc = (MultiVersionsProduct) productsMap.get(productName);
	    return pvc;
  } 

  public URL getProductFileUrl(Product product) {
    String jarPath = buildProductFileRelativePath(product);
    try {
      return jarPath != null ? new URL(storeRootDirURL, jarPath) : null;
    } catch (MalformedURLException e) {
      return null;
    }
  }

  public String getProductFilePath(Product product) {
    return JJarUtils.concatenatePath(prefixDirectory, buildProductFileRelativePath(product));
  }

  public String buildProductFileRelativePath(Product p) {
    if (p.getJarName() != null)
      return JJarUtils.concatenatePath(p.getDirectory(), p.getJarName());
    return null;
  }
  
  public boolean isProduction(){
	  return (prefixDirectory == null);
  }
  public Document getDocument(){
	  return parser.getDocument();
  }
  public URL getStoreRootDirURL(){
	  return storeRootDirURL;
  }
  
  private void load() throws IOException {
    loadLocalizations();
    if (repositoryDescriptorIsLocal && repositoryDescriptorFile != null && repositoryDescriptorFile.exists()) {
      parser = new JJarParser(repositoryDescriptorFile);
    } else if (repositoryDescriptorURL != null) {
      parser = new JJarParser(repositoryDescriptorURL);
    } else {
      return;
    }
    productsMap = parser.getProductVersionMap();
    prefixDirectory = parser.getPrefixDirectory();
  }

  private void loadLocalizations() throws MalformedURLException {
    //Add slash to the path if it is not there and create URL
    repositoryDescriptorURL = new URL(repositoryDescriptor);
    repositoryDescriptorIsLocal = FILE.equals(repositoryDescriptorURL.getProtocol());
    if (repositoryDescriptorIsLocal) {
      repositoryDescriptorFile = new File(repositoryDescriptorURL.getPath());
    }
    storeRootDirURL = new URL(storeRootDir);
  }

}