/*
 * $Id $
 * 
 * $Date: 2006/11/01 13:18:03 $ $Revision: 1.12 $ $Author: enes $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.runner;

import java.io.File;
import java.net.URL;

import cern.jjar.repository.Repository;
import cern.jjar.repository.RepositoryXML;
import cern.jjar.repository.RepositoryXMLStore;
import cern.jjar.utils.Logger;

/**
 * The parent of all runners
 * 
 * @author Lionel Mestre
 * @version $Id: AbstractJJarRunner.java,v 1.12 2006/11/01 13:18:03 enes Exp $
 */
abstract class AbstractJJarRunner implements JJarRunner {

  protected static final int LOG_LINE_LENGTH = 68;

  private final Logger logger;
  
  private boolean silent;
  
  private RepositoryElement[] repositoryElements;

  protected String defaultdependencyversion;
  
  public RepositoryXMLStore repositoryXMLStore;

  //
  // -- CONSTRUCTORS -----------------------------------------------
  //

  public AbstractJJarRunner() {
    this(null);
  }

  public AbstractJJarRunner(Logger logger) {
    this.logger = logger;
  }

  //
  // -- PUBLIC METHODS -----------------------------------------------
  //
  
  public void setRepositoryElements(RepositoryElement[] repositoryElements) {
    this.repositoryElements = repositoryElements;
  }

  public void setSilent(boolean silent) {
    this.silent = silent;
  }

  public boolean isSilent() {
    return silent;
  }
  
  /**
   * @param defaultdependencyversion The defaultdependencyversion to set.
   */
  public final void setDefaultdependencyversion(String defaultdependencyversion) {
    this.defaultdependencyversion = defaultdependencyversion;
  }

  //
  // -- PROTECTED METHODS -----------------------------------------------
  //
  
  protected Repository lookupRepository() {
    RepositoryXML newRepository = new RepositoryXML(defaultdependencyversion);
    RepositoryElement[] repositoryElements = getRepositoryElements();
    
    for (int i = 0; i < repositoryElements.length; i++) {
      try {
    	repositoryXMLStore = new RepositoryXMLStore(repositoryElements[i].getUrl());
    	newRepository.addStore(repositoryXMLStore);
      } catch (Exception e) {
        System.out.println("Cannot load " + repositoryElements[i] + (e.getMessage()));
      }
    }
    return newRepository;
  }
  
  protected RepositoryElement getRepositoryElement() {
    if (repositoryElements == null || repositoryElements.length == 0) {
      return null;
    }
    return repositoryElements[0];
  }
  
  protected RepositoryElement[] getRepositoryElements() {
    return repositoryElements;
  }
  
  protected static final String getFileName(URL url) {
    File f = new File(url.getPath());
    return f.getName();
  }

  protected void log(String msg) {
    if (silent) {
      return;
    }
    if (logger != null) {
      logger.log(msg);
    } else {
      System.out.println(msg);
    }
  }
}