/*
 * $Id $
 * 
 * $Date: 2004/12/06 13:33:40 $ $Revision: 1.3 $ $Author: lmestre $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.runner;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import cern.jjar.resources.Resources;
import cern.jjar.utils.Logger;
import cern.jjar.repository.Repository;

/**
 * @author gkruk
 */
public class UnregisterRunner extends AbstractJJarRunner {

  /* Name of the product */
  private String product;

  /* Version of the product */
  private String version;

  //
  // -- CONSTRUCTORS -----------------------------------------------
  //

  /* Creates a new instance of PublishRunner */
  public UnregisterRunner(Logger logger) {
    super(logger);
  }

  //
  // -- PUBLIC METHODS -----------------------------------------------
  //
  
  public void setProduct(String product) {
    this.product = product;
  }

  public void setVersion(String version) {
    this.version = version;
  }

  public Object run() throws Exception {
    checkParameters();
    File repositoryDir = new File(new URL(getRepositoryElement().getUrl()).getPath());
    unregisterProduct(repositoryDir);
    return null;
  }

  protected void unregisterProduct(File repositoryDir) throws TransformerException, IOException {
    Templates templates = Resources.getUnregisterProductTemplate();
    Transformer t = templates.newTransformer();
    t.setParameter("name", this.product);
    t.setParameter("version", this.version);

    File output = new File(repositoryDir, "_" + Repository.REPOSITORY_FILE_NAME);
    output.createNewFile();
    File repositoryFile = new File(repositoryDir, Repository.REPOSITORY_FILE_NAME);
    t.transform(new StreamSource(repositoryFile), new StreamResult(output));

    repositoryFile.delete();
    output.renameTo(repositoryFile);
  }

  private void checkParameters() throws Exception {
    if (getRepositoryElement() == null)
      throw new Exception("Remote repository has to be set!");
  }
  
  public static void main(String[] args) {
    try {
      UnregisterRunner runner = new UnregisterRunner(null);
      runner.setRepositoryElements(new RepositoryElement[] { 
          new RepositoryElement("file:////web5.cern.ch/test-netbeans/test/Release/dev")});
      runner.setProduct("bic-ui");
      runner.setVersion("0.1");
      //runner.setOnlydependencies(true);
      //runner.setKeepversion(true);
      //runner.setLocalprojectpriority(true);
      runner.run();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}