/*
 * $Id $
 * 
 * $Date: 2005/01/10 08:15:59 $ $Revision: 1.7 $ $Author: lmestre $
 * 
 * Copyright CERN, All Rights Reserved.
 */
package cern.jjar.utils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLConnection;

/**
 * bit moving support for JJar
 * 
 * @author Geir Magnusson Jr. (original idea from Jakarta)
 * @version $Id: Transport.java,v 1.7 2005/01/10 08:15:59 lmestre Exp $
 */
public class Transport {

  /*
   * This function may be used while releasing product on PCROPS. There is no
   * need to copy all jars into the local lib, but instead better is to make
   * links.
   */
  public static void makeLink(File source, String dest) throws IOException {
    Runtime.getRuntime().exec("ln -s " + source + " " + dest);
  }

  public static void fetchJar(File source, String dest) throws IOException {
    fetchJar(new FileInputStream(source), dest);
  }

  public static void fetchJar(URLConnection source, String dest) throws IOException {
    fetchJar(source.getInputStream(), dest);
  }

  private static void fetchJar(InputStream source, String dest) throws IOException {
    File destDir = new File(dest).getParentFile();
    destDir.mkdirs();
    BufferedInputStream bis = null;
    BufferedOutputStream bos = null;
    try {
      bis = new BufferedInputStream(source);
      bos = new BufferedOutputStream(new FileOutputStream(dest));
      fetch(bis, bos);
    } finally {
        if (bos != null)
          bos.close();
    }
  }

  private static void fetch(InputStream is, OutputStream os) throws IOException {
    byte[] buf = new byte[1024];
    int count = 0;
    do {
      os.write(buf, 0, count);
      count = is.read(buf, 0, buf.length);
    } while (count != -1);
  }
}
