package release.notification;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.plaf.basic.BasicColorChooserUI.PropertyHandler;
import javax.xml.transform.TransformerException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import release.notification.parsers.NotificationDOMParser;

public class SubscriptionHandler{
    
    private NotificationDOMParser domParser;
    protected Document document;
    
    protected static final String SUB_DIR = PropertiesHandler.getInstance().loadPropetry("subscriptions.descriptor.dir");
    protected static final String SUB_FILE_NAME = PropertiesHandler.getInstance().loadPropetry("subscriptions.descriptor.filename");
    
    public SubscriptionHandler(){
        try{
            System.out.println(SUB_DIR + SUB_FILE_NAME);
            domParser = new NotificationDOMParser(new File(SUB_DIR + SUB_FILE_NAME));
        }catch(IOException ioex){
            System.out.println("Couldn't read the subscription.xml file");            
        }
        document = domParser.getDocument();
    }
    
    protected Element findNode(XPath xpath, Element root, String expression) {

        try {
          Element element = (Element) xpath.evaluate(expression, root, XPathConstants.NODE);
          return element;
        } catch (XPathExpressionException xpex) {
          xpex.printStackTrace();
          System.out.println("xpath failed");
        } catch (Exception ex) {
          ex.printStackTrace();
        }
        return null;
    }
    
    protected NodeList findNodeList(XPath xpath, Element root, String expression) {

        try {
          NodeList nodeList = (NodeList) xpath.evaluate(expression, root, XPathConstants.NODESET);
          return nodeList;
        } catch (XPathExpressionException xpex) {
          xpex.printStackTrace();
          System.out.println("xpath failed");
        } catch (Exception ex) {
          ex.printStackTrace();
        }
        return null;
    }
    
    protected static final void saveSubscriptionFile(Document document, File outPutDir) throws TransformerException, IOException {
        File outputFile = new File(outPutDir, SUB_FILE_NAME);
        
        OutputStream outputStream = new FileOutputStream(outputFile); 
        
        OutputFormat format = new OutputFormat(document);
        format.setEncoding("UTF-8");
        format.setIndent(2);
        format.setLineWidth(500);
        format.setPreserveSpace(false);
        
        XMLSerializer seriOutput = new XMLSerializer(outputStream, format);
        seriOutput.serialize(document);
        
        outputStream.close();
    }
    
    protected boolean inputsVerified(String email, String[] products){
        
        if(!verifyEmail(email) || products.length==0){
            return false;
        }
        return true;
    }
    
    protected String processEmail(String email){
        String processedEmail;
        
        processedEmail = email.trim();
        processedEmail = processedEmail.toLowerCase();
        
        return processedEmail;
    }
    
    protected boolean verifyEmail(String email){
        
        if("*".equals(email)){
            return true;
        }
        
        //Set the email pattern string
        Pattern p = Pattern.compile(".+@.+\\.[a-z]+");

        //Match the given string with the pattern
        Matcher m = p.matcher(email);

        //check whether match is found 
        boolean matchFound = m.matches();

        if(matchFound){
            System.out.println("Valid Email Id.");
            return true;
        }else{
            System.out.println("Invalid Email Id.");
            return false;
        }
    }
} 