package release.notification.parsers;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException; 
import java.io.InputStream;
import java.net.URL;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class NotificationSAXParser extends DefaultHandler{
    
    public NotificationSAXParser(URL url, DefaultHandler handler) throws IOException{
        this(url.openStream(), handler);
    }
    
    public NotificationSAXParser(File file, DefaultHandler handler) throws IOException{
        this(new FileInputStream(file), handler);
    }
    
    public NotificationSAXParser(InputStream xmlInput, DefaultHandler handler) throws IOException{
        parse(xmlInput, handler);
    }
    
    private void parse(InputStream xmlInput, DefaultHandler handler){
        try {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            factory.setNamespaceAware(true);
            SAXParser parser = factory.newSAXParser();
            parser.parse(xmlInput, handler);
        } 
        catch (FactoryConfigurationError e) {
            System.out.println("unable to get a document builder factory");
        } 
        catch (ParserConfigurationException e) {
            System.out.println("saxParser was unable to be configured");
        }
        catch (SAXException e) {
            System.out.println("parsing error");
        } 
        catch (IOException e) {
            System.out.println("i/o error");
        }
    }
    
}