/*
 * Wrapper program for ant buildfile of Release tool.
 * 
 * It translates command line parameteres to ant properties defined
 * while running ant on the Release buildfile. 

 * This program is meant (and, hopefully, safe) to have a SUID bit set.
 *
*/

#include <stdio.h>
#include <sys/types.h>
#include <strings.h>
#include <stdarg.h>
#include <string.h>
#ifdef __hpux
	#define _INCLUDE_POSIX_SOURCE 
#endif
#include <pwd.h>
#include <unistd.h>
#include <stdlib.h>

#define ANT_COMMAND "ant -q -f release.xml"

#define ERROR(x)  { fprintf(stderr, "Release error: %s \n", x); \
		    _exit(1); }

/* declarations */
void parse_args(char **, char **);



/* prints program options */
void usage() {
	printf("Usage: ./release [-pr|-product] [-dp|-distpath] [-rep]" 
			" [-srcdir] [-src] [-v] [-nl] [-cvstag] [-ko] [-cf]"
			" [-default] [-validate] [-targets] [-systems] [-sv|-rb]\n\n");

	printf("-pr|-product		defines a name of a product\n"
		"-pr=<product_name>\n\n"
		
		"-dp|-distpath		defines distribution area path, if different from default\n"
		"-dp=<path_to_distribution_area>\n\n"

		"-rep			specified repository, if different from default\n"
		"-rep=<CVS repository>\n\n"

		"-srcdir	used only when source is directory (-src=dir)\n"
		"-srcdir=<directory>	<directory> will be used as a source of a product\n\n"
		
		"-src		specified source of a product as CVS, Razor or directory\n"
		"-src=<cvs,razor,dir>\n\n"

		"-v		version number to set on a product.\n"
		"-v=<version number>\n"
		"For setVersion target (-sv) - version to set an alias (PRO,PREVIOUS,NEXT) to.\n"
		"For rebuildProduct - version to be used for rebuild.\n\n"

		"-nl		Do not source .login file on operational account.\n\n"
		"-cvstag	Specified CVS tag for checkout of a product.\n"
		"-cvstar=<TAG>\n\n"
		"-ko		Keepoutput files from specified version number.\n"
		"-ko=<version>\n"
		"-ko		Will ask for keepoutput version.\n\n"
		"-cf		Take product.xml into account\n\n"
		"-default	Use defaults (default asnwers).\n\n"
		"-validate	Validate product.xml file when parsing. Only valid with -cf\n\n"
		"-targets	List of targets to execute.\n"
		"-targets=<target1,target2,..>\n\n"
		"-systems	List of systems to build on.\n"
		"-systems=<system1,system2,..>\n\n"
		"-sv		setVersion target. Sets alias (PRO, PREVIOUS or NEXT).\n"
		"-sv -as=<alias>	alias: PRO, PREVIOUS or NEXT.\n\n"
		"-rb		rebuildProduct, rebuild product.\n"
		"-rb=<version>\n\n");

	exit(0);
}

/* appends all the arguments of type 'char *' to the *base string */
/* last argument has to be NULL */

void append(char **base, ...) {
	va_list ap;
	char *arg = NULL, *x;
	unsigned int size = 0;
	
	x = *base;
	if (x) size += strlen(x);

	/* let's count the length of all argument first */
	/* realloc() costs too much to be used for each arg */
#ifdef __hpux
	va_start(ap, base);
#else
	va_start(ap, *base);
#endif
	arg = va_arg(ap, char *);
	while (arg) {
		size += strlen(arg);
		arg = va_arg(ap, char *);
 	}
	va_end(ap);


#ifdef __hpux
	*base = (char *) malloc(size);
	if (!*base) ERROR("realloc() failed\n");
	strcat(*base, x);
	free(x);
#else
/* under HP-UX this resulted in segfault in mallinfo() - I have no idea why */

	*base = (char*) realloc(*base, size);
#endif

	if (!*base) ERROR("realloc() failed\n");
	if (!x) memset(*base, '\0', size);

	/* once we have sufficient space - append the arguments */
#ifdef __hpux
	va_start(ap, base);
#else
	va_start(ap, *base);
#endif
	arg = va_arg(ap, char *);

	while (arg) {
		strcat(*base, arg);
		arg = va_arg(ap, char *);
	}		
	
	va_end(ap);	
}


/* MAIN :) */
int main(int argc, char **argv) {
	struct passwd *info;
	uid_t uid, euid;
	gid_t gid, egid;
	char *anthome  = getenv("ANT_HOME");
	char *command  = NULL;
#ifdef __hpux
	char env_str[128];
#endif

	uid  = getuid();	
	euid = geteuid();
	gid  = getgid();
	egid = getegid();
	info = getpwuid(uid);

#ifdef __hpux
	sprintf(env_str,"USER=%s", info->pw_name);
	putenv(env_str);
#else
	setenv("USER", info->pw_name, 1);
#endif

	append(&command, anthome, "/bin/", ANT_COMMAND, " ", NULL); 
	
	/* parse the arguments to ant properties */
	if (argc > 1) parse_args(&command, argv);

	setresuid(euid, euid, uid);
	setresgid(egid, egid, gid);
	umask(022);
	
	system(command);
	setresuid(uid, uid, uid);
	setresgid(gid, gid, gid);

  /* TODO: do properly memory cleanup */
	/* if (command) free(command); */
	
	return 0;
}


/* translates argument list args to ant properties */
void parse_args(char **cmd, char **args) {
	char *value = NULL;

	while (*++args) {

		/* repository */
		if (strstr(*args, "-rep=")  == *args) {
        value = index(*args, '=');
        ++value;
        append(cmd, " ", "-Drelease.src.rep=", value, NULL);
        continue; 
    }
                        

		/* product name options */
		if (strstr(*args, "-pr=") == *args ||
		    strstr(*args, "-product=") == *args) {
			value = index(*args, '=');
			++value;
			append(cmd, " ", "-Drelease.product.name=", value, NULL);
			continue;	
		}			

		/* path to distribution area */
		if (strstr(*args, "-dp=") == *args ||
			strstr(*args, "-distpath=") == *args){
			value = index(*args, '=');
			++value;
			append(cmd, " ", "-Drelease.areas.distribution=", value, NULL);
			continue;
		} 

		/* rebuild target */

		if (strstr(*args, "-rb") == *args) {
			value = index(*args, '=');
			if (value) {
				++value;
				append(cmd, " ", "-Drelease.product.version=", value, NULL);
			}  
			append(cmd, " ", "rebuildProduct", NULL);
			continue; 
		}

		/* source directory */
		if (strstr(*args, "-srcdir=") == *args ||
			strstr(*args, "-sourcedir=") == *args){
			append(cmd, " ", "-Drelease.product.source=dir", NULL);
			
			value = index(*args, '=');
			if (value) {		
				++value;
				append(cmd, " ", "-Dsrc.dir=", value, NULL);
			}
			continue;
		} 

		/* setversion target */
		if (strstr(*args, "-sv") == *args) {
			append(cmd, " ", "setVersion", NULL);
			continue;
		}
	
		
		if (strstr(*args, "-src=") == *args) {
			value = index(*args, '=');
			++value;
			append(cmd, " ", "-Drelease.product.source=", value, NULL);
			continue;
		}	

		if (strstr(*args, "-v=") == *args) {
			/*if (!strstr(*cmd, "setVersion"))
				ERROR("-v option can only be used together with -sv (setVersion target)");
                        */
			value = index(*args, '=');
			++value;
			append(cmd, " ", "-Drelease.product.version=", value, NULL);
		
			continue;
		}

		if (strstr(*args, "-as=") == *args) {
			value = index(*args, '=');
			++value;
			append(cmd, " ", "-Dwhich.version=", value, NULL);
			continue;
		}


		if (strstr(*args, "-nl") == *args) {
			append(cmd, " ", "-Dno.login=true", NULL);
		}

		if (strstr(*args, "-cvstag=") == *args) {
			value = index(*args, '=');
			++value;
			append(cmd, " ", "-Drelease.product.source=cvs", NULL);
			append(cmd, " ", "-Drelease.cvs.tag=", value,  NULL);

		}

		if (strstr(*args, "-ko") == *args) {
			value = index(*args, '=');
	    if (value) {
		    ++value;                            
		    append(cmd, " ", "-Drelease.keepoutput.version=", value, NULL);
	    }
			append(cmd, " ", "-Dkeepoutput=true", NULL);
			continue;
		}
		
		/* new option - for using product.xml file */
		if (strstr(*args, "-cf") == *args) {
			append(cmd, " ", "-Duse.product.xml=true", NULL);
	    continue;
		}
                
    if (strstr(*args, "-default") == *args) {
        append(cmd, " ", "-Drelease.usedefaults=true", NULL);
        continue;
    }

    /* validate against schema file */
    if (strstr(*args, "-validate") == *args) {
        append(cmd, " ", "-Drelease.validate=true", NULL);
        continue;
    }

    /* list of coma-separated targets */
    if (strstr(*args, "-targets") == *args) {
        value = index(*args, '=');
        if (value) {
                ++value;                            
                append(cmd, " ", "-Drelease.targets=", value, NULL);
        }
        continue;
    }

    /* list of comaseparated systems */
    if (strstr(*args, "-systems") == *args) {
        value = index(*args, '=');
        if (value) {
                ++value;                            
                append(cmd, " ", "-Dsystems=", value, NULL);
        }
        continue;
    }

	  /* razor options - not supported yet */
	  if (strstr(*args, "-th=")  == *args || 
	      strstr(*args, "-gr=") == *args)
	        ERROR("Razor source not implemented.");
	
/* TODO:
		 * -ks, -keepstruct ?
		 * -mo, -makeopts

			-help
	*/
		
		if (strstr(*args, "--help") == *args) {
			usage();
		}
	}

}

